<?php
// Nagios XI Operations Center Component --> Problem Overview
//
// Copyright (c) 2010 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: snmptranslate.inc.php 115 2010-08-16 16:15:26Z mguthrie $

//include the helper file
require_once(dirname(__FILE__).'/../componenthelper.inc.php');

// respect the name
$snmptranslate_component_name="snmptranslate";

// run the initialization function
snmptranslate_component_init();

////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////
	
function snmptranslate_component_init(){
	global $snmptranslate_component_name;

	// configuration file location
	$snmptt_filename="/etc/snmp/snmptt.conf";
	
	//boolean to check for latest version
	$versionok=snmptranslate_component_checkversion();
	
	//component description
	$desc=gettext("This page will allow an administrator to edit the Severity and/or Text for each defined type of SNMP Trap. (It updates the configuration stored in file ".$snmptt_filename.".)");
	
	if(!$versionok)
		$desc="<b>".gettext("Error: This component requires Nagios XI 2011R1 or later.")."</b>";
	
	if(!file_exists($snmptt_filename))
		$desc="<b>Error: Configuration file ".$snmptt_filename." does not exist. This component requires the installation of snmptt-1.3-3.nagios.noarch.rpm or equivalent.</b>";
	
	//all components require a few arguments to be initialized correctly.  
	$args=array(

		// need a name
		COMPONENT_NAME => $snmptranslate_component_name,
		COMPONENT_VERSION => '1.0', 
		COMPONENT_DATE => '12/31/2013',

		// informative information
		COMPONENT_AUTHOR => "Rod Payne, Kaleida Health.  (Components by Nagios Enterprises, LLC.)",
		COMPONENT_DESCRIPTION => $desc,
		COMPONENT_TITLE => "SNMP Trap Translation",

		);
	
	//register this component with XI 
	register_component($snmptranslate_component_name,$args);
	
	// register the addmenu function
	if($versionok)
		register_callback(CALLBACK_MENUS_INITIALIZED,'snmptranslate_component_addmenu');
	}
	

///////////////////////////////////////////////////////////////////////////////////////////
// MISC FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function snmptranslate_component_checkversion(){

	if(!function_exists('get_product_release'))
		return false;
	//requires greater than 2009R1.2
	if(get_product_release()<200)
		return false;

	return true;
	}
	
function snmptranslate_component_addmenu($arg=null){
	global $snmptranslate_component_name;
	//retrieve the URL for this component
	$urlbase=get_component_url_base($snmptranslate_component_name);

	// figure out where I'm going on the menu
	// - refer to /usr/local/nagiosxi/html/includes/utils-menu-inc.php for help with menu ID's	

	$mi=find_menu_item(MENU_ADMIN,"menu-admin-missingobjects","id");
	if($mi==null) //bail if I didn't find the above menu item 
		return;
		
	$order=grab_array_var($mi,"order","");  //extract this variable from the $mi array 
	if($order=="")
		return;
		
	$neworder=$order-0.1; //determine my menu order 

	//add this to the main home menu 
	add_menu_item(MENU_ADMIN,array(
		"type" => "link",
		"title" => gettext("SNMP Trap Translation"),
		"id" => "menu-admin-snmptranslate",
		"order" => $neworder,
		"opts" => array(
			//this is the page the menu will actually point to.
			//all of my actual component workings will happen on this script
			"href" => $urlbase."/snmptranslate.php",      
			)
		));

	}

?>