<?php
//
// Copyright (c) 2008-2009 Nagios Enterprises, LLC.  All rights reserved.
//
// Rod Payne - Kaleida Health - 31Dec2013
// This page will allow an administrator to edit the Severity and/or Text for each defined type of SNMP Trap.
//
// Built snmptranslate.php using Nagios "mobilecarriers.php" code as a template.
// Tested with Nagios XI 2012R2.5 and snmptt-1.3-3.
//

require_once(dirname(__FILE__).'/../../common.inc.php');

// initialization stuff
pre_init();

// start session
init_session();

// grab GET or POST variables 
grab_request_vars();

// check prereqs
check_prereqs();

// check authentication
check_authentication(false);

// only admins can access this page
if(is_admin()==false){
	echo $lstr['NotAuthorizedErrorText'];
	exit();
	}

// configuration file location
	$snmptt_filename="/etc/snmp/snmptt.conf";
	
// route request
route_request();


function route_request(){
	global $request;

	if(isset($request['update']))
		do_update_settings();
		
	show_settings();
	exit;
	}

	
function show_settings($error=false,$msg=""){
	global $request;
	global $lstr;
	global $snmptt_filename;
	
	// get variables submitted to us by our form

	$tt=grab_request_var("tt");
	if(is_array($tt)) {
		$ttinfo=$tt;
	} else {

	// get info from the snmptt configuration file
	
		$ttinfo=array();
		
		$snmptt_file=fopen($snmptt_filename, "r");
		if (!$snmptt_file) {
			$error=true;
			$msg="Could not open $snmptt_filename.";
		} else {
			$mib_name="Unknown";
			$within_long_desc=false;
			$long_desc="";
			while (!feof($snmptt_file)) {
				$line=fgets($snmptt_file);
				if ($within_long_desc==true) {
					if ((trim($line)=="EDESC") or (trim($line)=="xVariables:")) {
						$within_long_desc=false;
						array_push($ttinfo,array("oid"=>$event_oid,"mib"=>$mib_name,"mibfile"=>$mib_file,"event"=>$event_name,"severity"=>$event_level,"text"=>$exec_desc,"desc"=>$long_desc));
		// 				array_push($ttinfo,array("mib"=>$mib,"mibfile"=>$mib_file,"event"=>$event_name,"oid"=>$event_oid,"severity"=>$event_level,"text"=>$exec_desc,"desc"=>$long_desc));
					} else {
						$long_desc=$long_desc . htmlentities($line) . "<br>";
					}
				} else {
					$tokens=explode(" ",$line);
					$token=trim($tokens[0]);
					switch ($token) {
						case "MIB:":
							$mib_name=$tokens[1];
							$mib_file=$tokens[2];
							break;
						case "EVENT":
							$event_name=$tokens[1];
							$event_oid=$tokens[2];
							$event_level=strtoupper(trim($tokens[5]));
							switch ($event_level) {
								case "INFORMATIONAL":
									$event_level="NORMAL";
									break;
								case "SEVERE":
									$event_level="CRITICAL";
									break;
								case "MAJOR":
									$event_level="CRITICAL";
									break;
								case "MINOR":
									$event_level="WARNING";
									break;
							}
							break;
						case "EXEC":
							$quoted_strings=explode('"',$line);
							$exec_desc=$quoted_strings[11];
							break;
						case "SDESC":
							$long_desc="";
							$within_long_desc=true;
							break;
					}
				}
			}
			fclose($snmptt_file);

			sort($ttinfo);
		}
	}

	do_page_start(array("page_title"=>"SNMP Trap Translation"),true);
?>
	<h1>SNMP Trap Translation</h1>	
<?php
	display_message($error,false,$msg);
?>
	<p>
	This page will allow an administrator to edit the Severity and/or Text for each defined type of SNMP Trap.  
	(The configuration is stored in file <?php echo $snmptt_filename ?>.)
	</p>

	<form id="manageSnmpTranslationsForm" method="post" action="<?php echo htmlentities($_SERVER['PHP_SELF']);?>">
	
	<input type="hidden" name="update" value="1">
	<?php echo get_nagios_session_protector();?>
	
	<table>
	<thead>
	<tr><th>#</th>
		<th>OID</th>
		<th>MIB</th>
		<th>Event Name</th>
		<th>Severity</th>
		<th>Text</th>
	</tr>
	</thead>
	
	<tbody>
<?php
	$x=1;
	foreach($ttinfo as $ttarr){
		echo "<tr valign='top'>";
		echo "<td rowspan='2' align='center'>".($x)."</td>";
		echo "<td rowspan='2'><input type='text' name='tt[".$x."][oid]' value='".htmlentities($ttarr["oid"])."' size='22' readonly></td>";
		echo "<td><input type='text' name='tt[".$x."][mib]' value='".htmlentities($ttarr["mib"])."' size='16' readonly></td>";
		echo "<td><input type='text' name='tt[".$x."][event]' value='".htmlentities($ttarr["event"])."' size='30' readonly></td>";
		echo "<td rowspan='2'><select name='tt[".$x."][severity]'>";
			switch ($ttarr["severity"]) {
				case "CRITICAL":
					echo "<option value='NORMAL'>NORMAL</option><option value='WARNING'>WARNING</option><option value='CRITICAL' SELECTED>CRITICAL</option>";
					break;
				case "WARNING":
					echo "<option value='NORMAL'>NORMAL</option><option value='WARNING' SELECTED>WARNING</option><option value='CRITICAL'>CRITICAL</option>";
					break;
				default:
					echo "<option value='NORMAL' SELECTED>NORMAL</option><option value='WARNING'>WARNING</option><option value='CRITICAL'>CRITICAL</option>";
					break;
			}
		echo "</select></td>";
		echo "<td><input type='text' name='tt[".$x."][text]' value='".htmlentities($ttarr["text"])."' size='90'></td>";
		echo "</tr><tr valign='top'>";
		echo "<td colspan='2'>";
		if (array_key_exists("mibfile",$ttarr)) {
			echo htmlentities($ttarr["mibfile"]);
		} else {
			echo " ";
		}
		echo "</td><td>";
		if (array_key_exists("mibfile",$ttarr)) {
			echo $ttarr["desc"];
		} else {
			echo " ";
		}
		echo "</td></tr>";
		$x++;
		}
?>
	</tbody>
	</table>
	
	<div id="formButtons">
	<input type="submit" class="submitbutton" name="updateButton" value="<?php echo $lstr['UpdateSettingsButton'];?>" id="updateButton">
	<input type="submit" class="submitbutton" name="cancelButton" value="<?php echo $lstr['CancelButton'];?>" id="cancelButton">
	</div>
	

	<!--</fieldset>-->
	</form>
	
<?php

	do_page_end(true);
	exit();
	}


function do_update_settings(){
	global $request;
	global $lstr;
	global $snmptt_filename;
	
	// user pressed the cancel button
	if(isset($request["cancelButton"]))
		header("Location: /nagiosxi/admin/main.php");
	
	// check session
	check_nagios_session_protector();

	$errmsg=array();
	$errors=0;
	$updatemsgs=0;
	
	// get variables submitted to us
	$tt=grab_request_var("tt");

//	if ($ttarr=find_ttarr_from_web_form(".1.3.6.1.2.1.17.0.1", $tt)) {
//		print "<br>"; print_r($ttarr);
//		print "<br>mib=".$ttarr["mib"].", event=".$ttarr["event"];
//		print "<br>severity=".$ttarr["severity"].", text=".$ttarr["text"];
//	}
//	exit();
	
	// make sure we have requirements
	if(in_demo_mode()==true)
		$errmsg[$errors++]=$lstr['DemoModeChangeError'];
	if(!is_array($tt))
		$errmsg[$errors++]=gettext("Could not process request");
	foreach($tt as $ttid => $ttarr){
		$event_oid=grab_array_var($ttarr,"oid");
		$event_level=grab_array_var($ttarr,"severity");
		$exec_desc=grab_array_var($ttarr,"text");
					
		if($event_oid=="")
			$errmsg[$errors++]=gettext("Unique ID missing on line #").$ttid;
		if(trim($event_level)=="")
			$errmsg[$errors++]=gettext("Severity missing on line #").$ttid;
		if(trim($exec_desc)=="")
			$errmsg[$errors++]=gettext("Text missing on line #").$ttid;
		}
		
	// handle input errors
	if($errors>0) {
		show_settings(true,$errmsg);
	}

	// update configuration file (chown apache.nagios /etc/snmp/snmptt.conf ?)
		
	copy($snmptt_filename,$snmptt_filename.".update") or show_settings(true,"Could not copy $snmptt_filename to $snmptt_filename.update");
	
	$snmptt_source_file=fopen($snmptt_filename, "r") or show_settings(true,"Could not open $snmptt_filename");
	$snmptt_update_file=fopen($snmptt_filename.".update", "w") or show_settings(true,"Could not open $snmptt_filename.update");
	$mib_name="Unknown";
	$within_long_desc=false;
	$long_desc="";
	while (!feof($snmptt_source_file)) {
		$line=fgets($snmptt_source_file);
		if ($within_long_desc==true) {
			if (trim($line)=="EDESC") {
				$within_long_desc=false;
			}
			fwrite($snmptt_update_file,$line);
		} else {
			$tokens=explode(" ",$line);
			$token=trim($tokens[0]);
			switch ($token) {
				case "MIB:":
					$mib_name=$tokens[1];
					$mib_file=$tokens[2];
					break;
				case "EVENT":
					$event_name=$tokens[1];
					$event_oid=$tokens[2];
					$event_level=strtoupper(trim($tokens[5]));
					if ($ttarr=find_ttarr_from_web_form($event_oid, $tt)) {
						if ($event_level<>$ttarr["severity"]) {
							$updatemsg[$updatemsgs++]=$ttarr["mib"]." event ".$ttarr["event"]." severity changed from '".$event_level."' to '".$ttarr["severity"]."'.";
							$tokens[5]=$ttarr["severity"];
							$line=implode(" ",$tokens)."\n";
						}						
					}
					break;
				case "EXEC":
					$quoted_strings=explode('"',$line);
					$exec_desc=$quoted_strings[11];
					if ($ttarr) {
						if ($exec_desc<>$ttarr["text"]) {
							$updatemsg[$updatemsgs++]=$ttarr["mib"]." event ".$ttarr["event"]." text changed from '".$exec_desc."' to '".$ttarr["text"]."'.";
							$line=str_replace($exec_desc,$ttarr["text"],$line);
						}						
					}
					break;
				case "SDESC":
					$within_long_desc=true;
					break;
			}
			fwrite($snmptt_update_file,$line);
		}
	}
	fclose($snmptt_source_file);
	fclose($snmptt_update_file);

	rename($snmptt_filename,$snmptt_filename.".backup") or show_settings(true,"Could not rename $snmptt_filename to $snmptt_filename.backup");
	rename($snmptt_filename.".update",$snmptt_filename) or show_settings(true,"Could not rename $snmptt_filename.update to $snmptt_filename");
	
	$updatemsg[$updatemsgs++]="SNMP Traps Translation configuration file updated.";	
	
	// log it
	send_to_audit_log(gettext("User updated SNMP traps translation settings"),AUDITLOGTYPE_CHANGE);
	
//	$updatemsg[$updatemsgs++]=shell_exec('service snmptt restart');
	
	show_settings(false,$updatemsg);
	}

function find_ttarr_from_web_form($search, $tt)
{	
	if ($searchresult=array_searchRecursive($search, $tt)) {
		return $tt[$searchresult[0]];
	} else {
		return false;
	}
}

/** 
 * Searches haystack for needle and returns an array of the key path if it is found in the (multidimensional) array, FALSE otherwise. 
 * 
 * mixed array_searchRecursive ( mixed needle, array haystack [, bool strict[, array path]] ) 
 * http://www.webdeveloper.com/forum/showthread.php?224930-Searching-a-multidimensional-array
 */ 
 
function array_searchRecursive( $needle, $haystack, $strict=false, $path=array() ) 
{ 
    if( !is_array($haystack) ) { 
        return false; 
    } 
 
    foreach( $haystack as $key => $val ) { 
        if( is_array($val) && $subPath = array_searchRecursive($needle, $val, $strict, $path) ) { 
            $path = array_merge($path, array($key), $subPath); 
            return $path; 
        } elseif( (!$strict && $val == $needle) || ($strict && $val === $needle) ) { 
            $path[] = $key; 
            return $path; 
        } 
    } 
    return false;  
} 	
?>